@@ -4,8 +4,9 @@
  *
  * Copyright (C) (C) 2003  Vladimir Oleynik  <dzo@simtreas.ru>
  *
- * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
+ * Licensed under GPLv2 or later, see file LICENSE in this source tree.
  */
+#include "libbb.h"
 
 /*
    This function make special for recursive actions with usage
@@ -13,9 +14,7 @@
    and skipping "." and ".." directory entries
 */
 
-#include "libbb.h"
-
-char *concat_subpath_file(const char *path, const char *f)
+char* FAST_FUNC concat_subpath_file(const char *path, const char *f)
 {
 	if (f && DOT_OR_DOTDOT(f))
 		return NULL;
