@@ -8,16 +8,12 @@
  */
 #include "libbb.h"
 
-/* Find out if the last character of a string matches the one given.
- * Don't underrun the buffer if the string length is 0.
- */
+/* Find out if the last character of a string matches the one given */
 char* FAST_FUNC last_char_is(const char *s, int c)
 {
-	if (s && *s) {
-		size_t sz = strlen(s) - 1;
-		s += sz;
-		if ( (unsigned char)*s == c)
-			return (char*)s;
-	}
-	return NULL;
+	if (!s[0])
+		return NULL;
+	while (s[1])
+		s++;
+	return (*s == (char)c) ? (char *) s : NULL;
 }
