@@ -10,44 +10,36 @@
  * Licensed under GPLv2, see file LICENSE in this source tree.
  */
 //config:config CRYPTPW
-//config:	bool "cryptpw"
+//config:	bool "cryptpw (14 kb)"
 //config:	default y
 //config:	help
-//config:	  Encrypts the given password with the crypt(3) libc function
-//config:	  using the given salt.
+//config:	Encrypts the given password with the crypt(3) libc function
+//config:	using the given salt.
 //config:
 //config:config MKPASSWD
-//config:	bool "mkpasswd"
+//config:	bool "mkpasswd (15 kb)"
 //config:	default y
 //config:	help
-//config:	  Encrypts the given password with the crypt(3) libc function
-//config:	  using the given salt. Debian has this utility under mkpasswd
-//config:	  name. Busybox provides mkpasswd as an alias for cryptpw.
+//config:	Encrypts the given password with the crypt(3) libc function
+//config:	using the given salt. Debian has this utility under mkpasswd
+//config:	name. Busybox provides mkpasswd as an alias for cryptpw.
 
-//applet:IF_CRYPTPW(APPLET(cryptpw, BB_DIR_USR_BIN, BB_SUID_DROP))
-//                   APPLET_ODDNAME:name      main     location        suid_type     help
-//applet:IF_MKPASSWD(APPLET_ODDNAME(mkpasswd, cryptpw, BB_DIR_USR_BIN, BB_SUID_DROP, cryptpw))
+//applet:IF_CRYPTPW( APPLET_NOEXEC(cryptpw,  cryptpw, BB_DIR_USR_BIN, BB_SUID_DROP, cryptpw))
+//                   APPLET_NOEXEC:name      main     location        suid_type     help
+//applet:IF_MKPASSWD(APPLET_NOEXEC(mkpasswd, cryptpw, BB_DIR_USR_BIN, BB_SUID_DROP, cryptpw))
 
 //kbuild:lib-$(CONFIG_CRYPTPW) += cryptpw.o
 //kbuild:lib-$(CONFIG_MKPASSWD) += cryptpw.o
 
 //usage:#define cryptpw_trivial_usage
-//usage:       "[OPTIONS] [PASSWORD] [SALT]"
+//usage:       "[-P FD] [-m TYPE] [-S SALT] [PASSWORD] [SALT]"
 /* We do support -s, we just don't mention it */
 //usage:#define cryptpw_full_usage "\n\n"
 //usage:       "Print crypt(3) hashed PASSWORD\n"
-//usage:	IF_LONG_OPTS(
-//usage:     "\n	-P,--password-fd N	Read password from fd N"
-/* //usage:  "\n	-s,--stdin		Use stdin; like -P0" */
-//usage:     "\n	-m,--method TYPE	"CRYPT_METHODS_HELP_STR
-//usage:     "\n	-S,--salt SALT"
-//usage:	)
-//usage:	IF_NOT_LONG_OPTS(
 //usage:     "\n	-P N	Read password from fd N"
 /* //usage:  "\n	-s	Use stdin; like -P0" */
 //usage:     "\n	-m TYPE	"CRYPT_METHODS_HELP_STR
 //usage:     "\n	-S SALT"
-//usage:	)
 
 #include "libbb.h"
 
@@ -80,8 +72,8 @@ ENVIRONMENT
     A list of options which will be evaluated before the ones
     specified on the command line.
 BUGS
-    This programs suffers of a bad case of featuritis.
-    ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+    This program suffers of a bad case of featuritis.
+    ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 
 Very true...
 
@@ -106,14 +98,15 @@ int cryptpw_main(int argc UNUSED_PARAM, char **argv)
 		"salt\0"        Required_argument "S"
 		"method\0"      Required_argument "m"
 	;
-	applet_long_options = mkpasswd_longopts;
 #endif
 	fd = STDIN_FILENO;
 	opt_m = CONFIG_FEATURE_DEFAULT_PASSWD_ALGO;
 	opt_S = NULL;
 	/* at most two non-option arguments; -P NUM */
-	opt_complementary = "?2";
-	getopt32(argv, "sP:+S:m:a:", &fd, &opt_S, &opt_m, &opt_m);
+	getopt32long(argv, "^" "sP:+S:m:a:" "\0" "?2",
+			mkpasswd_longopts,
+			&fd, &opt_S, &opt_m, &opt_m
+	);
 	argv += optind;
 
 	/* have no idea how to handle -s... */
@@ -132,8 +125,8 @@ int cryptpw_main(int argc UNUSED_PARAM, char **argv)
 	if (!password) {
 		/* Only mkpasswd, and only from tty, prompts.
 		 * Otherwise it is a plain read. */
-		password = (ENABLE_MKPASSWD && isatty(STDIN_FILENO) && applet_name[0] == 'm')
-			? bb_ask_stdin("Password: ")
+		password = (ENABLE_MKPASSWD && applet_name[0] == 'm' && isatty(STDIN_FILENO))
+			? bb_ask_noecho_stdin("Password: ")
 			: xmalloc_fgetline(stdin)
 		;
 		/* may still be NULL on EOF/error */
