@@ -377,11 +377,11 @@ static int parse_module(module_info *info, const char *pathname)
 	return info->open_read_failed;
 }
 
-static FAST_FUNC int fileAction(const char *pathname,
-		struct stat *sb UNUSED_PARAM,
-		void *modname_to_match,
-		int depth UNUSED_PARAM)
+static FAST_FUNC int fileAction(struct recursive_state *state,
+		const char *pathname,
+		struct stat *sb UNUSED_PARAM)
 {
+	const char *modname_to_match = state->userData;
 	int cur;
 	const char *fname;
 	bool is_remove = (ENABLE_RMMOD && ONLY_APPLET)
@@ -751,8 +751,7 @@ static int process_module(char *name, const char *cmdline_options)
 			ACTION_RECURSE, /* flags */
 			fileAction, /* file action */
 			NULL, /* dir action */
-			name, /* user data */
-			0 /* depth */
+			name /* user data */
 		);
 		dbg1_error_msg("dirscan complete");
 		/* Module was not found, or load failed, or is_remove */
