@@ -32,7 +32,10 @@ struct filter_t {
 	int state;
 	int unused_only;
 	inet_prefix pfx;
+	/* Misnomer. Does not mean "flushed N something" */
+	/* More like "no_of_flush_commands_constructed_by_print_neigh()" */
 	int flushed;
+	/* Flush cmd buf. If !NULL, print_neigh() constructs flush commands in it */
 	char *flushb;
 	int flushp;
 	int flushe;
@@ -45,8 +48,8 @@ typedef struct filter_t filter_t;
 
 static int flush_update(void)
 {
-	if (rtnl_send(G_filter.rth, G_filter.flushb, G_filter.flushp) < 0) {
-		bb_perror_msg("can't send flush request");
+	if (rtnl_send_check(G_filter.rth, G_filter.flushb, G_filter.flushp) < 0) {
+		bb_simple_perror_msg("can't send flush request");
 		return -1;
 	}
 	G_filter.flushp = 0;
@@ -299,12 +302,10 @@ static int FAST_FUNC ipneigh_list_or_flush(char **argv, int flush)
 		G_filter.rth = &rth;
 
 		while (round < MAX_ROUNDS) {
-			if (xrtnl_wilddump_request(&rth, G_filter.family, RTM_GETNEIGH) < 0) {
-				bb_perror_msg_and_die("can't send dump request");
-			}
+			xrtnl_wilddump_request(&rth, G_filter.family, RTM_GETNEIGH);
 			G_filter.flushed = 0;
 			if (xrtnl_dump_filter(&rth, print_neigh, NULL) < 0) {
-				bb_perror_msg_and_die("flush terminated");
+				bb_simple_perror_msg_and_die("flush terminated");
 			}
 			if (G_filter.flushed == 0) {
 				if (round == 0)
@@ -324,11 +325,11 @@ static int FAST_FUNC ipneigh_list_or_flush(char **argv, int flush)
 	ndm.ndm_family = G_filter.family;
 
 	if (rtnl_dump_request(&rth, RTM_GETNEIGH, &ndm, sizeof(struct ndmsg)) < 0) {
-		bb_perror_msg_and_die("can't send dump request");
+		bb_simple_perror_msg_and_die("can't send dump request");
 	}
 
 	if (xrtnl_dump_filter(&rth, print_neigh, NULL) < 0) {
-		bb_error_msg_and_die("dump terminated");
+		bb_simple_error_msg_and_die("dump terminated");
 	}
 
 	return 0;
